
#include "defs.h"
#include <stdio.h>
#include <stdlib.h>

/*********************************************************/
/* Environment Dynamics Function: Construct base XORing  */
/* masks for (partially) cyclical changing environments  */
/*********************************************************/
void constructBaseXORMasks(void) {
  int i, j, numOnes;
  int *randIndex = new int[lchrom]; 

  // create a random permutation of 0 to lchrom-1
  randomSequence(randIndex, lchrom, lchrom);

  // initialize the baseXORMasks
  for (i=0; i<totalBaseStates; ++i)
    for (j=0; j<lchrom; ++j) baseXORMasks[i][j] = 0;

  // configure the baseXORMasks
  numOnes = (int)(changeDegree * lchrom);
  for (i=0; i<totalBaseStates; ++i) {
    for (j=0; j<numOnes; ++j)
      baseXORMasks[i][randIndex[numOnes*i+j]] = 1; 
  }

  delete [] randIndex;
}

/*******************************************************/
/* Environment Dynamics Function: Incremental changing */
/*******************************************************/
void calcXORMask(void) {
  int i, numOnes, baseStateIdx;
  int *intermMask = new int[lchrom];  // intermediate mask

  for (i=0; i<lchrom; ++i) intermMask[i] = 0; 

  if ( bRandRatio == true)
    changeDegree = randomNumber(0.0, 1.0);
  numOnes = (int)(changeDegree * lchrom);

  switch(changeMode) {
    case 1: // cyclical change
      baseStateIdx = ((int)(currGenNo/changeSpeed))%totalBaseStates;
      for (i=0; i<lchrom; ++i)  // copy the relevant base XORing mask 
        intermMask[i] = baseXORMasks[baseStateIdx][i];
      break;
    case 2: // partially cyclical change
      baseStateIdx = ((int)(currGenNo/changeSpeed))%totalBaseStates;
      for (i=0; i<lchrom; ++i) // copy the relevant base XORing mask
        intermMask[i] = baseXORMasks[baseStateIdx][i];

      for (i=0; i<lchrom; ++i)
        if (randomNumber(0.0, 1.0) < pBaseMut)
          intermMask[i] = 1 - intermMask[i];
      break;
    case 3: // random non-cyclical change
      int *index2 = new int[numOnes];
      randomSequence(index2, numOnes, lchrom);
      for (i=0; i<numOnes; ++i) // create numOnes 1's in intermMask[]
        intermMask[index2[i]] = 1;
      delete [] index2;
      break;
  }

  // integrate intermMask into xorMask
  for (i=0; i<lchrom; ++i) 
    xorMask[i] ^= intermMask[i];

  delete [] intermMask;
}

/****************************************************************************
 *			Change the problem				    *
 ****************************************************************************/
void changeEnvironment( void ) {
  calcXORMask();

  // Calculate new fitness
  for (int i = 0; i < popsize; i++) 
    popcurr.ind[i].fitness = calcFitness(popcurr.ind[i].chromosome);

  statistics( &popcurr );
}

