#include "defs.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <fstream>

/*****************************************************************************
 *				 Statistics				     *
 *****************************************************************************/
void statistics(population *pop) {
  int j;

  pop->sum_fitness = pop->ind[0].fitness; // sum of fitness in the population 
  pop->max_fitness = pop->ind[0].fitness; // max fitness in the population 
  pop->best_individual = 0;	// best individual in the population
  pop->best_individual_2 = 0;	// second best individual in the population

  for (j=1; j<popsize; j++) {
    pop->sum_fitness += pop->ind[j].fitness;
    if (pop->ind[j].fitness >= pop->max_fitness) {
      pop->max_fitness = pop->ind[j].fitness ; 
      pop->best_individual_2 = pop->best_individual;
      pop->best_individual = j;			
    }
  }
  pop->mean_fitness = pop->sum_fitness/popsize;

  // store elite from previous generation
  if (currGenNo > 0) 
    elite_prev_pop = individual_best;

  // store best individuals from current population
  individual_best = pop->ind[pop->best_individual];
  individual_best_2 = pop->ind[pop->best_individual_2];

  // initialize elite_prev_pop to the best of the initial population   
  if (currGenNo == 0) 
    elite_prev_pop = individual_best;

  // Save the data
  log_mean_fitness[currGenNo] = pop->mean_fitness;
  log_best_fitness[currGenNo] = pop->max_fitness;
}

/***************************************************************/
/* calculate the diversity of current population               */
/***************************************************************/
void calcPopDiversity(population *pop) {
  int i, j, k, hammingDist;
  double popDiversity = 0.0; 
  for (i = 0; i<popsize; ++i) 
    for (j = 0; j<popsize; ++j) {
      hammingDist = 0;
      for (k = 0; k<lchrom; ++k) {
        if (pop->ind[i].chromosome[k] != pop->ind[j].chromosome[k])
          hammingDist += 1;
      }
      popDiversity += (double)hammingDist;
    }

  popDiversity /= (double)(lchrom*popsize*(popsize-1));

  // Save the data
  log_mean_diversity[currGenNo] = popDiversity;
}

void saveFitnessResult(int num_exec) {
  FILE *logPopBest;  // output file for pop best fitness

  char bestfilename[60];
  if (bRandRatio == false) 
    sprintf(bestfilename, "PopBest_GA%d_F%d_%d_%4.2f_%d_%4.2f_%4.3f_%4.2f.txt",
            gaMode, functNo, changeMode, changeDegree, changeSpeed, ratio_immigrants,
            pImmigMut, pBaseMut);
  else 
    sprintf(bestfilename, "PopBest_GA%d_F%d_%d_Rand_%d_%4.2f_%4.3f_%4.2f.txt",
            gaMode, functNo, changeMode,changeSpeed, ratio_immigrants,
            pImmigMut, pBaseMut);

  if ((logPopBest = fopen(bestfilename,"a")) == NULL) { exit(2); }
  fprintf(logPopBest, "%2d ", num_exec);
  for (int i=0; i<maxgen+1; i++) 
    fprintf(logPopBest, "%d ", (int)log_best_fitness[i]);
  fprintf(logPopBest, "\n");
  fclose(logPopBest);
}

void saveDiversityResult(int num_exec) {
  FILE *logPopDiv;   // output file for pop diversity

  char divfilename[60];
  if (bRandRatio == false) {
    sprintf(divfilename, "PopDiv_GA%d_F%d_%d_%4.2f_%d_%4.2f_%4.3f_%4.2f.txt",
       gaMode, functNo, changeMode, changeDegree, changeSpeed, ratio_immigrants,
       pImmigMut, pBaseMut);
  } else {
    sprintf(divfilename, "PopDiv_GA%d_F%d_%d_Rand_%d_%4.2f_%4.3f_%4.2f.txt",
       gaMode, functNo, changeMode,changeSpeed, ratio_immigrants,
       pImmigMut, pBaseMut);
  }

  if ((logPopDiv = fopen(divfilename,"a")) == NULL) { exit(3); }
  fprintf(logPopDiv, "%2d ", num_exec);
  for (int i=0; i<maxgen+1; i++) 
    fprintf(logPopDiv, "%4.3f ", log_mean_diversity[i]);
  fprintf(logPopDiv, "\n");
  fclose(logPopDiv);
}
